#!/bin/bash

# This script looks for a BOOST installation in a standard location.
# If a single BOOST installation is found, it prints out the full path of
# the (static) library and returns with exit code 0.
# If none is found or several are found, it prints out an error message
# and returns with a non-zero exit code.

##################################################################
# Use find to search through various standard directories.
# NB look through all directories, even if a GMP has already been found.

# List of directories under which boost hpp dir is normally found.
STD_BOOST_HPPDIRS="/usr/lib64:/usr/local:/usr/lib:/opt/local/include:/sw/include:/usr/sfw/include:"

IFS=":"
HPPBOOSTPATHS=hppboost-paths
/bin/rm -rf $HPPBOOSTPATHS
for directory in $STD_BOOST_HPPDIRS
do
  if [ -d "$directory" ]
  then
    find "$directory" -name  boost   -print >> $HPPBOOSTPATHS  2> /dev/null
  fi
done

if [ \! -s $HPPBOOSTPATHS ]
then
  # Did not find any plausible BOOST installation, so return empty handed.
  echo "No BOOST installation found; looked inside $STD_BOOST_HPPDIRS"
  /bin/rm $HPPBOOSTPATHS
  exit 1
fi

# Slightly odd call to wc is to avoid it printing out the file name.
if [ `wc -l < $HPPBOOSTPATHS` -ne 1 ]
then
  echo "*** Found multiple BOOST libraries ***"
  cat $HPPBOOSTPATHS
  /bin/rm $HPPBOOSTPATHS
  exit 2
fi

# We have found a single directory called boost do a couple of quick
# sanity checks before declaring our search successful...
BOOST_HPP=`cat $HPPBOOSTPATHS`
/bin/rm $HPPBOOSTPATHS
BOOST_DIR=`dirname "$BOOST_HPP"`
BOOST_HDR="$BOOST_DIR/boost"
if [ -d "$BOOST_HDR" -a -r "$BOOST_HDR" ]
then
  # We've found a plausible boost dir and (probably) its corresponding boost.
  echo "$BOOST_HDR"
  exit 0
fi

# Get here probably only if there's a damaged BOOST installation.
echo "Trouble reading boost dir $BOOST_HDR."
exit 3
