#ifndef CoCoA_RandomSource_H
#define CoCoA_RandomSource_H

//   Copyright (c)  2010  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.
//   -------------------------------------------------------------------------

#include "CoCoA/MachineInteger.H"
#include "CoCoA/ZZ.H"

#include "gmp.h"
#include <iosfwd>
//using std::ostream;

namespace CoCoA
{

  class RandomSource
  {
  public:
    explicit RandomSource(const MachineInteger& seed = 1);
    RandomSource(const RandomSource& copy);
    RandomSource& operator=(const RandomSource& rhs);
    ~RandomSource();
    friend bool RandomBool(RandomSource& src);
    friend long RandomLong(RandomSource& src);
    friend long RandomLong(RandomSource& src, const MachineInteger& lwb, const MachineInteger& upb);
    friend ZZ RandomZZ(RandomSource& src, const MachineInteger& lwb, const MachineInteger& upb);
    friend ZZ RandomZZ(RandomSource& src, const MachineInteger& lwb, const ZZ& upb);
    friend ZZ RandomZZ(RandomSource& src, const ZZ& lwb, const MachineInteger& upb);
    friend ZZ RandomZZ(RandomSource& src, const ZZ& lwb, const ZZ& upb);
    friend std::ostream& operator<<(std::ostream& out, const RandomSource& src);
  private:  // data members
//     std::size_t myBoolCounter;
//     std::size_t myLongCounter;
//     std::size_t myZZCounter;
    unsigned long mySeed;
    gmp_randstate_t myState;
//    std::bitset<ourBufferBits> myBitBuffer;
  };

} // end of namespace CoCoA



// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/RandomSource.H,v 1.1 2010/12/17 16:03:15 abbott Exp $
// $Log: RandomSource.H,v $
// Revision 1.1  2010/12/17 16:03:15  abbott
// Added new class RandomSource.
//
//

#endif
