#ifndef CoCoA_RandomZZStream_H
#define CoCoA_RandomZZStream_H

//   Copyright (c)  2010  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/MachineInteger.H"
#include "CoCoA/ZZ.H"

#include <cstddef>
//using std::size_t;
#include <iosfwd>
//using std::ostream;
#include "gmp.h"

namespace CoCoA
{

  class RandomZZStream
  {
  public:
    RandomZZStream(const MachineInteger& lwb, const MachineInteger& upb, const MachineInteger& seed = 1);
    RandomZZStream(const MachineInteger& lwb, const ZZ& upb, const MachineInteger& seed = 1);
    RandomZZStream(const ZZ& lwb, const MachineInteger& upb, const MachineInteger& seed = 1);
    RandomZZStream(const ZZ& lwb, const ZZ& upb, const MachineInteger& seed = 1);
    RandomZZStream(const RandomZZStream& copy);
    RandomZZStream& operator=(const RandomZZStream& rhs);
    ~RandomZZStream();
    const ZZ& operator*() const;
    RandomZZStream& operator++();
    RandomZZStream operator++(int);
    std::size_t myIndex() const;
  private:
    void myGenValue(); // fills myValue with a newly generated random value.
  private: // data members
    ZZ myLwb;   // morally const, changed only by assignment
    ZZ myUpb;   // morally const, changed only by assignment
    ZZ myRange; // morally const, changed only by assignment
    ZZ myValue;
    std::size_t myCounter;
    unsigned long mySeed;
    gmp_randstate_t myState;
    friend std::ostream& operator<<(std::ostream& out, const RandomZZStream& RLS);
  };

  const ZZ& sample(RandomZZStream& RLS);
  std::ostream& operator<<(std::ostream& out, const RandomZZStream& RLS);

  //---------------------------------------------------------------------------
  // Inline functions.

  inline const ZZ& RandomZZStream::operator*() const
  {
    return myValue;
  }


  inline RandomZZStream& RandomZZStream::operator++()
  {
    ++myCounter;
    myGenValue();
    return *this;
  }


  inline const ZZ& sample(RandomZZStream& RLS)
  {
    return *++RLS; // advance then get value
  }

} // end of namespace CoCoA



// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/RandomZZStream.H,v 1.1 2010/06/29 15:17:02 abbott Exp $
// $Log: RandomZZStream.H,v $
// Revision 1.1  2010/06/29 15:17:02  abbott
// New class RandomZZStream
//
//

#endif
