//   Copyright (c)  2010  Anna Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/DenseMatrix.H"
#include "CoCoA/MatrixSpecial.H"
#include "CoCoA/PolyRing.H" // for jacobian
#include "CoCoA/error.H"
#include "CoCoA/matrix.H"
#include "CoCoA/tmp.H" // for jacobian

#include <iostream>
using std::ostream;
//#include <cstddef>
using std::size_t;
//#include <vector>
using std::vector;

namespace CoCoA
{


  //---------  jacobian matrix

  matrix jacobian_aux(const PolyRing& P, const std::vector<RingElem>& polys, const std::vector<RingElem>& inds)
  {
    matrix ans = NewDenseMat(P, polys.size(), NumIndets(P));
    for (size_t i=0; i < polys.size(); ++i)
      for (size_t j=0; j < inds.size(); ++j)
        SetEntry(ans, i,j, deriv(polys[i], inds[j]));
    return ans;
  }


  matrix jacobian(const std::vector<RingElem>& polys)
  {
    if (polys.size()==0) CoCoA_ERROR(ERR::BadArg, "jacobian(polys)");
    if (!HasUniqueOwner(polys)) CoCoA_ERROR(ERR::MixedRings, "jacobian(polys)");
    const PolyRing P = AsPolyRing(owner(polys[0]));
    return jacobian_aux(P, polys, indets(P));
  }
  

  matrix jacobian(const std::vector<RingElem>& polys, const std::vector<RingElem>& inds)
  {
    if (inds.size()==0 && (polys.size()==0)) CoCoA_ERROR(ERR::BadArg, "jacobian");
    if (!HasUniqueOwner(inds)) CoCoA_ERROR(ERR::MixedRings, "jacobian");
    if (!HasUniqueOwner(polys)) CoCoA_ERROR(ERR::MixedRings, "jacobian");
    if (inds.size()==0)
      return NewDenseMat(AsPolyRing(owner(polys[0])), polys.size(), 0);
    const PolyRing P = AsPolyRing(owner(inds[0]));
    if (polys.size()==0) return NewDenseMat(P, 0, inds.size());
    if (owner(polys[0]) != P) CoCoA_ERROR(ERR::MixedRings, "jacobian");
    return jacobian_aux(P, polys, inds);
  }
  
// VandermondeMatrix
// HessianMatrix
// HilbertMatrix
// HilbertInverseMatrix
// ToeplitzMatrix
// WronskianMatrix
// SylvesterMatrix


} // end of namespace CoCoA


// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/MatrixSpecial.C,v 1.2 2011/02/10 15:46:21 bigatti Exp $
// $Log: MatrixSpecial.C,v $
// Revision 1.2  2011/02/10 15:46:21  bigatti
// -- removed debug printing
//
// Revision 1.1  2011/02/10 15:30:14  bigatti
// -- first import: jacobian
//
