//   Copyright (c) 2010 Giovanni Lagorio (lagorio@disi.unige.it)
//
//   This file is part of the source of CoCoALib, the CoCoA Library.
//
//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.

#include "CoCoALibSupplement.H"

namespace CoCoA {

  RingElem LPP_forC5(ConstRefRingElem f)
  {
    ring R = owner(f);
    if (!IsPolyRing(R)) CoCoA_ERROR(ERR::NotElemPolyRing, "LPP_forC5");
    if (IsSparsePolyRing(R)) return monomial(AsSparsePolyRing(R), 1, LPP(f));
    PolyRing Rx = AsPolyRing(R);
    return IndetPower(Rx, 0, deg(f));
  }


  RingElem LM_forC5(ConstRefRingElem f)
  {
    ring R = owner(f);
    if (!IsPolyRing(R)) CoCoA_ERROR(ERR::NotElemPolyRing, "LM_forC5");
    if (IsSparsePolyRing(R)) return monomial(AsSparsePolyRing(R), LC(f), LPP(f));
    PolyRing Rx = AsPolyRing(R);
    return CoeffEmbeddingHom(Rx)(LC(f))*IndetPower(Rx, 0, deg(f));    
  }

  namespace
  {
    RingElem CoeffOfTermSparse(ConstRefRingElem f, ConstRefPPMonoidElem pp)
    {
      for (SparsePolyIter itf=BeginIter(f); !IsEnded(itf); ++itf)
        if (PP(itf) == pp) return coeff(itf);
      return zero(CoeffRing(AsPolyRing(owner(f))));
    }
    
    std::vector<RingElem> CoefficientsDense(ConstRefRingElem f)
    {
      std::vector<RingElem> v;
      DenseUPolyRing P = AsDenseUPolyRing(owner(f));
      for (size_t i=0; i<P->myDegPlus1(raw(f)); ++i)
        if (!IsZero(coeff(f,i))) v.push_back(coeff(f,i));
      return v;
    }
    
    std::vector<RingElem> CoefficientsSparse(ConstRefRingElem f)
    {
      std::vector<RingElem> v;
      for (SparsePolyIter itf=BeginIter(f); !IsEnded(itf); ++itf)
        v.push_back(coeff(itf));
      return v;
    }
    
    std::vector<RingElem> MonomialsDense(ConstRefRingElem f)
    {
      std::vector<RingElem> v;
      DenseUPolyRing P = AsDenseUPolyRing(owner(f));
      for (size_t i=0; i<P->myDegPlus1(raw(f)); ++i)
        if (!IsZero(coeff(f,i))) v.push_back(monomial(P, coeff(f,i), i));
      return v;
    }
    
    std::vector<RingElem> MonomialsSparse(ConstRefRingElem f)
    {
      std::vector<RingElem> v;
      SparsePolyRing P = AsSparsePolyRing(owner(f));
      for (SparsePolyIter itf=BeginIter(f); !IsEnded(itf); ++itf)
        v.push_back(monomial(P, coeff(itf), PP(itf)));
      return v;
    }
    
    std::vector<RingElem> SupportDense(ConstRefRingElem f)
    {
      std::vector<RingElem> v;
      DenseUPolyRing P = AsDenseUPolyRing(owner(f));
      for (size_t i=0; i<P->myDegPlus1(raw(f)); ++i)
        if (!IsZero(coeff(f,i))) v.push_back(monomial(P, one(CoeffRing(P)), i));
      return v;
    }

    std::vector<RingElem> SupportSparse(ConstRefRingElem f)
    {
      std::vector<RingElem> v;
      SparsePolyRing P = AsSparsePolyRing(owner(f));
      for (SparsePolyIter itf=BeginIter(f); !IsEnded(itf); ++itf)
        v.push_back(monomial(P, one(CoeffRing(P)), PP(itf)));
      return v;
    }  
  }
  

  RingElem CoeffOfTerm_forC5(ConstRefRingElem f, ConstRefRingElem t)
  {
    ring R = owner(f);
    if (!IsPolyRing(R)) CoCoA_ERROR(ERR::NotElemPolyRing, "CoeffOfTerm_forC5");
    if (!IsMonomial(t)) CoCoA_ERROR(ERR::BadArg, "CoeffOfTerm_forC5");
    if (!IsOne(LC(t))) CoCoA_ERROR(ERR::BadArg, "CoeffOfTerm_forC5");
    if (IsSparsePolyRing(R))
    {
      return CoeffOfTermSparse(f, LPP(t));
    }
    return coeff(f, deg(t));
  }
  
  
  std::vector<RingElem> coefficients_forC5(ConstRefRingElem f)
  {
    ring R = owner(f);
    if (!IsPolyRing(R)) CoCoA_ERROR(ERR::NotElemPolyRing, "coefficients_forC5");
    if (IsSparsePolyRing(R))  return CoefficientsSparse(f);
    return CoefficientsDense(f);
  }
  
  
  std::vector<RingElem> monomials_forC5(ConstRefRingElem f)
  {
    ring R = owner(f);
    if (!IsPolyRing(R)) CoCoA_ERROR(ERR::NotElemPolyRing, "monomials_forC5");
    if (IsSparsePolyRing(R))  return MonomialsSparse(f);
    return MonomialsDense(f);
  }
  
  
  std::vector<RingElem> support_forC5(ConstRefRingElem f)
  {
    ring R = owner(f);
    if (!IsPolyRing(R)) CoCoA_ERROR(ERR::NotElemPolyRing, "support_forC5");
    if (IsSparsePolyRing(R))  return SupportSparse(f);
    return SupportDense(f);
  }


  std::vector<ZZ> wdeg_forC5(ConstRefRingElem f)
  {
    degree d = wdeg(f);
    std::vector<ZZ> v;
    for (size_t i=0; i<GradingDim(d); ++i)  v.push_back(d[i]);
    return v;
  }


  ZZ NextPrime_forC5(ZZ N)
  {
    if (N <= 0) CoCoA_ERROR(ERR::NotNonNegative, "NextPrime_forC5");
    unsigned long n;
    if (!IsConvertible(n, N)) CoCoA_ERROR(ERR::ArgTooBig, "NextPrime_forC5");
    unsigned long ans = NextPrime(n);
    if (ans == 0) CoCoA_ERROR(ERR::ArgTooBig, "NextPrime_forC5");
    return ZZ(ans);
  }

  factorization<long> SmoothFactor_forC5(const ZZ& N, const ZZ& TrialLimit)
  {
    unsigned long n, tl;
    if (!IsConvertible(n, N)) CoCoA_ERROR(ERR::ArgTooBig, "SmoothFactor_forC5");
    if (!IsConvertible(tl, TrialLimit)) CoCoA_ERROR(ERR::ArgTooBig, "SmoothFactor_forC5");
    return SmoothFactor(n, tl);
  }

  QQ CpuTime_forC5()
  { //return ConvertTo<QQ>(CpuTime()); 
    QQ res;
    IsConvertible(res, CpuTime());
    return res;
  }
  

  matrix NewDenseMat_forC5(const ring& R, const ZZ& NR, const ZZ& NC)
  {
    unsigned long r,c;
    if (!IsConvertible(r, NR)) CoCoA_ERROR(ERR::BadRowIndex, "NewDenseMat_forC5");
    if (!IsConvertible(c, NC)) CoCoA_ERROR(ERR::BadColIndex, "NewDenseMat_forC5");
    return NewDenseMat(R, r, c);
  }
  

  void SetEntry_forC5(MatrixView& M, const ZZ& I, const ZZ& J, ConstRefRingElem x)
  {
    unsigned long i,j;
    if (!IsConvertible(i, I)) CoCoA_ERROR(ERR::BadRowIndex, "SetEntry_forC5");
    if (!IsConvertible(j, J)) CoCoA_ERROR(ERR::BadColIndex, "SetEntry_forC5");
    SetEntry(M, i-1, j-1, x);
  }
  

  MatrixView IdentityMat_forC5(const ring& R, const ZZ& N)
  {
    unsigned long n;
    if (!IsConvertible(n, N)) CoCoA_ERROR(ERR::BadMatrixSize, "IdentityMat_forC5");
    return IdentityMat(R, n);
  }
  

  matrix transposed_forC5(ConstMatrixView M)
  { return NewDenseMat(transpose(M)); }
  

}

